@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2010-2012 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2010-2013 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -336,7 +336,7 @@ cd_util_idle_loop_quit_cb (gpointer user_data)
 {
 	GMainLoop *loop = (GMainLoop *) user_data;
 	g_main_loop_quit (loop);
-	return FALSE;
+	return G_SOURCE_REMOVE;
 }
 /**
  * cd_util_sensor_cap_to_string:
@@ -387,6 +387,34 @@ cd_util_sensor_cap_to_string (CdSensorCap sensor_cap)
 		 * http://en.wikipedia.org/wiki/Plasma_display */
 		return _("Plasma");
 	}
+	if (sensor_cap == CD_SENSOR_CAP_LCD_CCFL) {
+		/* TRANSLATORS: this is the display technology,
+		 * where LCD stands for 'Liquid Crystal Display'
+		 * and CCFL stands for 'Cold Cathode Fluorescent Lamp' */
+		return _("LCD CCFL");
+	}
+	if (sensor_cap == CD_SENSOR_CAP_LCD_RGB_LED) {
+		/* TRANSLATORS: this is the display technology where
+		 * RGB stands for 'Red Green Blue' and LED stands for
+		 * 'Light Emitted Diode' */
+		return _("LCD RGB LED");
+	}
+	if (sensor_cap == CD_SENSOR_CAP_WIDE_GAMUT_LCD_CCFL) {
+		/* TRANSLATORS: this is the display technology, where
+		 * wide gamut means the display primaries are much
+		 * better than normal consumer monitors */
+		return _("Wide Gamut LCD CCFL");
+	}
+	if (sensor_cap == CD_SENSOR_CAP_WIDE_GAMUT_LCD_RGB_LED) {
+		/* TRANSLATORS: this is the display technology */
+		return _("Wide Gamut LCD RGB LED");
+	}
+	if (sensor_cap == CD_SENSOR_CAP_LCD_WHITE_LED) {
+		/* TRANSLATORS: this is the display technology, where
+		 * white means the color of the backlight, i.e. not
+		 * RGB LED */
+		return _("LCD White LED");
+	}
 	/* TRANSLATORS: this an unknown display technology */
 	return _("Unknown");
 }
@@ -997,6 +1025,8 @@ cd_util_sensor_set_options (CdUtilPrivate *priv, gchar **values, GError **error)
 {
 	CdSensor *sensor;
 	gboolean ret = TRUE;
+	gchar *endptr = NULL;
+	gdouble val;
 	GHashTable *options = NULL;
 	GPtrArray *array = NULL;
 	guint i;
@@ -1025,9 +1055,19 @@ cd_util_sensor_set_options (CdUtilPrivate *priv, gchar **values, GError **error)
 		goto out;
 	}
 
-	/* prepare options for each sensor */
+	/* prepare options for each sensor
+	 * NOTE: we're guessing the types here */
 	options = g_hash_table_new (g_str_hash, g_str_equal);
-	g_hash_table_insert (options, values[0], g_variant_new_string (values[1]));
+	val = g_ascii_strtod (values[1], &endptr);
+	if (endptr == NULL || endptr[0] == '\0') {
+		g_hash_table_insert (options,
+				     values[0],
+				     g_variant_new_double (val));
+	} else {
+		g_hash_table_insert (options,
+				     values[0],
+				     g_variant_new_string (values[1]));
+	}
 
 	for (i = 0; i < array->len; i++) {
 		sensor = g_ptr_array_index (array, i);
@@ -1543,61 +1583,21 @@ out:
 }
 
 /**
- * cd_util_profile_set_qualifier:
+ * cd_util_profile_set_property:
  **/
 static gboolean
-cd_util_profile_set_qualifier (CdUtilPrivate *priv, gchar **values, GError **error)
+cd_util_profile_set_property (CdUtilPrivate *priv, gchar **values, GError **error)
 {
 	CdProfile *profile = NULL;
 	gboolean ret = TRUE;
 
-	if (g_strv_length (values) < 2) {
+	if (g_strv_length (values) < 3) {
 		ret = FALSE;
 		g_set_error_literal (error,
 				     1, 0,
 				     "Not enough arguments, "
-				     "expected profile path, qualifier "
-				     "e.g. '/org/profile/foo epson.rgb.300dpi'");
-		goto out;
-	}
-
-	/* check is valid object path */
-	if (!g_variant_is_object_path (values[0])) {
-		ret = FALSE;
-		g_set_error (error,
-			     1, 0,
-			     "Not a valid object path: %s",
-			     values[0]);
-		goto out;
-	}
-
-	profile = cd_profile_new_with_object_path (values[0]);
-	ret = cd_profile_set_qualifier_sync (profile, values[1],
-					     NULL, error);
-	if (!ret)
-		goto out;
-out:
-	if (profile != NULL)
-		g_object_unref (profile);
-	return ret;
-}
-
-/**
- * cd_util_profile_set_filename:
- **/
-static gboolean
-cd_util_profile_set_filename (CdUtilPrivate *priv, gchar **values, GError **error)
-{
-	CdProfile *profile = NULL;
-	gboolean ret = TRUE;
-
-	if (g_strv_length (values) < 2) {
-		ret = FALSE;
-		g_set_error_literal (error,
-				     1, 0,
-				     "Not enough arguments, "
-				     "expected profile path, filename "
-				     "e.g. '/org/profile/foo bar.icc'");
+				     "expected profile path key value "
+				     "e.g. '/org/profile/foo qualifier RGB.Matte.300dpi'");
 		goto out;
 	}
 
@@ -1615,8 +1615,11 @@ cd_util_profile_set_filename (CdUtilPrivate *priv, gchar **values, GError **erro
 	ret = cd_profile_connect_sync (profile, NULL, error);
 	if (!ret)
 		goto out;
-	ret = cd_profile_set_filename_sync (profile, values[1],
-					    NULL, error);
+	ret = cd_profile_set_property_sync (profile,
+					    values[1],
+					    values[2],
+					    NULL,
+					    error);
 	if (!ret)
 		goto out;
 out:
@@ -2154,15 +2157,10 @@ main (int argc, char *argv[])
 		     _("Deletes a profile"),
 		     cd_util_delete_profile);
 	cd_util_add (priv->cmd_array,
-		     "profile-set-qualifier",
+		     "profile-set-property",
 		     /* TRANSLATORS: command description */
-		     _("Sets the profile qualifier"),
-		     cd_util_profile_set_qualifier);
-	cd_util_add (priv->cmd_array,
-		     "profile-set-filename",
-		     /* TRANSLATORS: command description */
-		     _("Sets the profile filename"),
-		     cd_util_profile_set_filename);
+		     _("Sets extra properties on the profile"),
+		     cd_util_profile_set_property);
 	cd_util_add (priv->cmd_array,
 		     "device-set-model",
 		     /* TRANSLATORS: command description */
