@@ -31,7 +31,6 @@
 
 #include "cd-debug.h"
 #include "cd-state.h"
-#include "cd-lcms-helpers.h"
 #include "cd-session.h"
 
 typedef struct {
@@ -1099,14 +1098,13 @@ out:
 static gboolean
 cd_main_set_profile_metadata (CdMainPrivate *priv, GError **error)
 {
-	cmsHANDLE dict_md = NULL;
-	cmsHPROFILE lcms_profile = NULL;
+	CdIcc *icc;
 	gboolean ret;
 	gchar *brightness_str = NULL;
-	gchar *data = NULL;
 	gchar *profile_fn = NULL;
 	gchar *profile_path = NULL;
-	gsize len;
+	GError *error_local = NULL;
+	GFile *file = NULL;
 
 	/* get profile */
 	profile_fn = g_strdup_printf ("%s.icc", priv->basename);
@@ -1115,91 +1113,71 @@ cd_main_set_profile_metadata (CdMainPrivate *priv, GError **error)
 					 NULL);
 
 	/* open profile */
-	ret = g_file_get_contents (profile_path, &data, &len, error);
+	icc = cd_icc_new ();
+	file = g_file_new_for_path (profile_path);
+	ret = cd_icc_load_file (icc,
+				file,
+				CD_ICC_LOAD_FLAGS_NONE,
+				priv->cancellable,
+				error);
 	if (!ret)
 		goto out;
-	lcms_profile = cmsOpenProfileFromMem (data, len);
-	if (lcms_profile == NULL) {
-		ret = FALSE;
-		g_set_error (error,
-			     CD_SESSION_ERROR,
-			     CD_SESSION_ERROR_FAILED_TO_OPEN_PROFILE,
-			     "failed to open profile %s",
-			     profile_path);
-		goto out;
-	}
 
 	/* add DICT data */
-	dict_md = cmsDictAlloc (NULL);
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_CMF_PRODUCT,
-			       "colord");
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_CMF_BINARY,
-			       "colord-session");
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_CMF_VERSION,
-			       PACKAGE_VERSION);
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_DATA_SOURCE,
-			       CD_PROFILE_METADATA_DATA_SOURCE_CALIB);
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_LICENSE,
-			       "CC0");
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_QUALITY,
-			       cd_profile_quality_to_string (priv->quality));
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_MAPPING_DEVICE_ID,
-			       cd_device_get_id (priv->device));
-	_cmsDictAddEntryAscii (dict_md,
-			       CD_PROFILE_METADATA_MEASUREMENT_DEVICE,
-			       cd_sensor_kind_to_string (cd_sensor_get_kind (priv->sensor)));
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_CMF_PRODUCT,
+			     "colord");
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_CMF_BINARY,
+			     "colord-session");
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_CMF_VERSION,
+			     PACKAGE_VERSION);
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_DATA_SOURCE,
+			     CD_PROFILE_METADATA_DATA_SOURCE_CALIB);
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_LICENSE,
+			     "CC0");
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_QUALITY,
+			     cd_profile_quality_to_string (priv->quality));
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_MAPPING_DEVICE_ID,
+			     cd_device_get_id (priv->device));
+	cd_icc_add_metadata (icc,
+			     CD_PROFILE_METADATA_MEASUREMENT_DEVICE,
+			     cd_sensor_kind_to_string (cd_sensor_get_kind (priv->sensor)));
 	if (priv->screen_brightness > 0) {
 		brightness_str = g_strdup_printf ("%u", priv->screen_brightness);
-		_cmsDictAddEntryAscii (dict_md,
-				       CD_PROFILE_METADATA_SCREEN_BRIGHTNESS,
-				       brightness_str);
+		cd_icc_add_metadata (icc,
+				     CD_PROFILE_METADATA_SCREEN_BRIGHTNESS,
+				     brightness_str);
 		g_free (brightness_str);
 	}
-	ret = cmsWriteTag (lcms_profile, cmsSigMetaTag, dict_md);
-	if (!ret) {
-		g_set_error_literal (error,
-				     CD_SESSION_ERROR,
-				     CD_SESSION_ERROR_FAILED_TO_SAVE_PROFILE,
-				     "cannot initialize dict tag");
-		goto out;
-	}
-
-	/* write profile id */
-	ret = cmsMD5computeID (lcms_profile);
-	if (!ret) {
-		g_set_error (error,
-			     CD_SESSION_ERROR,
-			     CD_SESSION_ERROR_INTERNAL,
-			     "failed to compute profile id for %s",
-			     profile_path);
-		goto out;
-	}
 
 	/* save file */
-	ret = cmsSaveProfileToFile (lcms_profile, profile_path);
+	ret = cd_icc_save_file (icc,
+				file,
+				CD_ICC_SAVE_FLAGS_NONE,
+				priv->cancellable,
+				&error_local);
 	if (!ret) {
 		g_set_error (error,
 			     CD_SESSION_ERROR,
 			     CD_SESSION_ERROR_FAILED_TO_SAVE_PROFILE,
-			     "failed to save profile to %s",
-			     profile_path);
+			     "failed to save profile to %s: %s",
+			     profile_path, error_local->message);
+		g_error_free (error_local);
 		goto out;
 	}
 out:
 	g_free (profile_fn);
 	g_free (profile_path);
-	g_free (data);
-	if (dict_md != NULL)
-		cmsDictFree (dict_md);
-	if (lcms_profile != NULL)
-		cmsCloseProfile (lcms_profile);
+	if (icc != NULL)
+		g_object_unref (icc);
+	if (file != NULL)
+		g_object_unref (file);
 	return ret;
 }
 
