@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2011-2012 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2011-2013 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -61,6 +61,8 @@
  * OUT: [1:retval][1:cmd][1:color_select]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_COLOR_SELECT			0x01
 
@@ -73,6 +75,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_COLOR_SELECT			0x02
 
@@ -85,6 +89,8 @@
  * OUT: [1:retval][1:cmd][1:multiplier_value]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_MULTIPLIER			0x03
 
@@ -97,6 +103,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_MULTIPLIER			0x04
 
@@ -109,6 +117,8 @@
  * OUT: [1:retval][1:cmd][2:integral_time]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_INTEGRAL_TIME		0x05
 
@@ -121,6 +131,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_INTEGRAL_TIME		0x06
 
@@ -133,6 +145,8 @@
  * OUT: [1:retval][1:cmd][2:major][2:minor][2:micro]
  *
  * This command is available in bootloader and firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_FIRMWARE_VERSION		0x07
 
@@ -155,6 +169,8 @@
  * OUT: [1:retval][1:cmd][2*9:matrix_value][1:types][23:description]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_CALIBRATION			0x09
 
@@ -167,6 +183,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_CALIBRATION			0x0a
 
@@ -179,6 +197,8 @@
  * OUT: [1:retval][1:cmd][4:serial_number]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_SERIAL_NUMBER		0x0b
 
@@ -191,6 +211,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_SERIAL_NUMBER		0x0c
 
@@ -203,6 +225,8 @@
  * OUT: [1:retval][1:cmd][1:led_state]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_LEDS				0x0d
 
@@ -219,6 +243,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_LEDS				0x0e
 
@@ -231,18 +257,24 @@
  * OUT: [1:retval][1:cmd][2:red][2:green][2:blue]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_DARK_OFFSETS			0x0f
 
 /**
  * CH_CMD_SET_DARK_OFFSETS:
  *
- * Set the dark offsets.
+ * Set the dark offsets for use when taking samples when the measure
+ * mode is FREQUENCY. When the measure mode is DURATION these values
+ * have no effect.
  *
  * IN:  [1:cmd][2:red][2:green][2:blue]
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_DARK_OFFSETS			0x10
 
@@ -255,6 +287,8 @@
  * OUT: [1:retval][1:cmd][60:owner-name]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_OWNER_NAME			0x11
 
@@ -267,6 +301,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_OWNER_NAME			0x12
 
@@ -279,6 +315,8 @@
  * OUT: [1:retval][1:cmd][60:owner-email]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_OWNER_EMAIL			0x13
 
@@ -291,6 +329,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_OWNER_EMAIL			0x14
 
@@ -303,6 +343,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_WRITE_EEPROM			0x20
 
@@ -311,10 +353,12 @@
  *
  * Take a raw reading.
  *
- * IN:  [1:cmd]
+ * IN:  [1:cmd][1:sensor-kind]
  * OUT: [1:retval][1:cmd][4:count]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_TAKE_READING_RAW			0x21
 
@@ -326,10 +370,12 @@
  *
  * This command is useful if you want to do an ambient reading.
  *
- * IN:  [1:cmd]
+ * IN:  [1:cmd][1:sensor-kind]
  * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_TAKE_READINGS			0x22
 
@@ -343,13 +389,96 @@
  * If the calibration index > %CH_CALIBRATION_MAX then the calibration
  * map is used to find the default calibration index to use.
  *
- * IN:  [1:cmd][2:calibration-index]
+ * If the calibration index is set to %CH_CALIBRATION_SPECTRAL then the
+ * spectral hardware is used if it is available. The CIE 1931 luminosity
+ * function data is used by default.
+ *
+ * IN:  [1:cmd][2:calibration-index][1:sensor-kind]
  * OUT: [1:retval][1:cmd][4:red][4:green][4:blue]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_TAKE_READING_XYZ			0x23
 
+/**
+ * CH_CMD_TAKE_READING_SPECTRAL:
+ *
+ * Takes a spectral reading putting %CH_CCD_SPECTRAL_RESOLUTION uint16_t values
+ * (typically 7296 bytes) in SRAM.
+ * The values are scaled from 0x0000 to 0x03ff and correspond to the actual
+ * signal amplitude once Vref- and Vref+ have been taken into account.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][4:addr]
+ *
+ * This command is only available in firmware mode.
+ *
+ * Since: 0.1.31
+ **/
+#define	CH_CMD_TAKE_READING_SPECTRAL		0x50
+
+/**
+ * CH_CMD_GET_ADC_CALIBRATION_POS:
+ *
+ * Gets the value of the Vref+ ADC adjustment.
+ * @vref is a packed float.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][4:vref]
+ *
+ * This command is only available in firmware mode.
+ *
+ * Since: 0.1.31
+ **/
+#define	CH_CMD_GET_ADC_CALIBRATION_POS		0x51
+
+/**
+ * CH_CMD_GET_ADC_CALIBRATION_NEG:
+ *
+ * Gets the value of the Vref- ADC adjustment.
+ * @vref is a packed float.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][4:vref]
+ *
+ * This command is only available in firmware mode.
+ *
+ * Since: 0.1.31
+ **/
+#define	CH_CMD_GET_ADC_CALIBRATION_NEG		0x52
+
+/**
+ * CH_CMD_GET_CCD_CALIBRATION:
+ *
+ * Gets the spectral indexes for the RGB calibration which are values between 0
+ * and %CH_CCD_SPECTRAL_RESOLUTION pointing to the dominant peak for the color.
+ * Red is nominally 625nm, green 520nm and blue 465nm.
+ *
+ * IN:  [1:cmd]
+ * OUT: [1:retval][1:cmd][2:red-idx][2:green-idx][2:blue-idx]
+ *
+ * This command is only available in firmware mode.
+ *
+ * Since: 0.1.31
+ **/
+#define	CH_CMD_GET_CCD_CALIBRATION		0x53
+
+/**
+ * CH_CMD_SET_CCD_CALIBRATION:
+ *
+ * Sets the spectral indexes for the RGB calibration.
+ *
+ * IN:  [1:cmd][2:red-idx][2:green-idx][2:blue-idx]
+ * OUT: [1:retval][1:cmd]
+ *
+ * This command is only available in firmware mode.
+ *
+ * Since: 0.1.31
+ **/
+#define	CH_CMD_SET_CCD_CALIBRATION		0x54
+
 /**
  * CH_CMD_RESET:
  *
@@ -359,6 +488,8 @@
  * OUT: [1:retval][1:cmd] (but with success the device will disconnect)
  *
  * This command is available in bootloader and firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_RESET				0x24
 
@@ -371,6 +502,8 @@
  * OUT: [1:retval][1:cmd][1:checksum][1-60:data]
  *
  * This command is only available in bootloader mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_READ_FLASH			0x25
 
@@ -385,6 +518,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in bootloader mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_ERASE_FLASH			0x29
 
@@ -398,6 +533,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in bootloader mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_WRITE_FLASH			0x26
 
@@ -410,6 +547,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in bootloader mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_BOOT_FLASH			0x27
 
@@ -436,6 +575,8 @@
  *
  * This command is available in bootloader and firmware mode, although
  * different values of @success are permitted in each.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_FLASH_SUCCESS		0x28
 
@@ -448,6 +589,8 @@
  * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_PRE_SCALE			0x2c
 
@@ -460,6 +603,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_PRE_SCALE			0x2d
 
@@ -472,6 +617,8 @@
  * OUT: [1:retval][1:cmd][4:scale]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_POST_SCALE			0x2a
 
@@ -484,6 +631,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_POST_SCALE			0x2b
 
@@ -508,6 +657,8 @@
  * OUT: [1:retval][1:cmd][6*2:types]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_CALIBRATION_MAP		0x2e
 
@@ -520,6 +671,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_CALIBRATION_MAP		0x2f
 
@@ -530,13 +683,16 @@
  *
  * The hardware versions are as follows:
  * 0x00		= Pre-production hardware
- * 0x01		= First generation hardware
- * 0x02-0x0f	= Reserved for future use
+ * 0x01		= ColorHug
+ * 0x02		= ColorHug Spectro
+ * 0x03-0x0f	= Reserved for future use
  *
  * IN:  [1:cmd]
  * OUT: [1:retval][1:cmd][1:hw_version]
  *
  * This command is available in bootloader and firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_HARDWARE_VERSION		0x30
 
@@ -557,6 +713,8 @@
  * OUT: [1:retval][1:cmd][30:reading_array]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_TAKE_READING_ARRAY		0x31
 
@@ -576,6 +734,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_PCB_ERRATA			0x32
 
@@ -588,6 +748,8 @@
  * OUT: [1:retval][1:cmd][2:pcb_errata]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_PCB_ERRATA			0x33
 
@@ -602,6 +764,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_REMOTE_HASH			0x34
 
@@ -615,6 +779,8 @@
  * OUT: [1:retval][1:cmd][20:sha1_hash]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_REMOTE_HASH			0x35
 
@@ -629,6 +795,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SET_MEASURE_MODE			0x36
 
@@ -641,6 +809,8 @@
  * OUT: [1:retval][1:cmd][1:measure_mode]
  *
  * This command is available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_MEASURE_MODE			0x37
 
@@ -653,6 +823,8 @@
  * OUT: [1:retval][1:cmd][1-60:data]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_READ_SRAM			0x38
 
@@ -665,6 +837,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_WRITE_SRAM			0x39
 
@@ -677,6 +851,8 @@
  * OUT: [1:retval][1:cmd][4:temperature]
  *
  * This command is only available in firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_GET_TEMPERATURE			0x3b
 
@@ -690,6 +866,8 @@
  * OUT: [1:retval][1:cmd]
  *
  * This command is available in bootloader and firmware mode.
+ *
+ * Since: 0.1.29
  **/
 #define	CH_CMD_SELF_TEST			0x40
 
@@ -706,9 +884,13 @@
 /* where the custom firmware is stored */
 #define CH_EEPROM_ADDR_RUNCODE			0x4000
 
+/* the number of useful samples from the the CCD */
+#define CH_CCD_SPECTRAL_RESOLUTION		3648
+
 /* although each calibration can be stored in 60 bytes,
  * we use a full 64 byte block */
 #define	CH_CALIBRATION_MAX			64	/* so finishes at device params */
+#define	CH_CALIBRATION_SPECTRAL			0xffff
 #define	CH_CALIBRATION_DESCRIPTION_LEN		23	/* 60 - (9*4) - 1 */
 
 /* the supported calibration types bitfield */
@@ -732,11 +914,12 @@
 #define	CH_FLASH_RECONNECT_TIMEOUT		5000	/* ms */
 
 /* calibration remapping contants */
-#define	CH_CALIBRATION_INDEX_FACTORY_ONLY		0x00
+#define	CH_CALIBRATION_INDEX_FACTORY_ONLY	0x00
 #define	CH_CALIBRATION_INDEX_LCD		(CH_CALIBRATION_MAX + 0)
 #define	CH_CALIBRATION_INDEX_CRT		(CH_CALIBRATION_MAX + 1)
 #define	CH_CALIBRATION_INDEX_PROJECTOR		(CH_CALIBRATION_MAX + 2)
 #define	CH_CALIBRATION_INDEX_LED		(CH_CALIBRATION_MAX + 3)
+#define	CH_CALIBRATION_INDEX_MAX		0x06
 
 /* which color to select */
 typedef enum {
@@ -750,7 +933,7 @@ typedef enum {
 typedef enum {
 	CH_STATUS_LED_GREEN	= 1 << 0,
 	CH_STATUS_LED_RED	= 1 << 1,
-	CH_STATUS_LED_BLUE	= 1 << 2
+	CH_STATUS_LED_BLUE	= 1 << 2	/* Since: 0.1.29 */
 } ChStatusLed;
 
 /* what frequency divider to use */
@@ -805,11 +988,19 @@ typedef enum {
 	CH_MEASURE_MODE_DURATION
 } ChMeasureMode;
 
+/* the sensor to use */
+typedef enum {
+	CH_SENSOR_KIND_MAIN,
+	CH_SENSOR_KIND_AMBIENT,
+	CH_SENSOR_KIND_CCD,
+	CH_SENSOR_KIND_LAST
+} ChSensorKind;
+
 /* any problems with the PCB */
 typedef enum {
 	CH_PCB_ERRATA_NONE		= 0,
 	CH_PCB_ERRATA_SWAPPED_LEDS	= 1 << 0,
-	CH_PCB_ERRATA_NO_WELCOME	=  1 << 1,
+	CH_PCB_ERRATA_NO_WELCOME	= 1 << 1,
 	CH_PCB_ERRATA_LAST		= 1 << 2
 } ChPcbErrata;
 
